# Temporal Source Cover Implementation

## Prerequisites

### Required

- dotnet SDK 5.0 or later (installers/binaries for Linux, macOS and Windows available at https://dotnet.microsoft.com/download/dotnet)

### Recommended

- Visual Studio 2019 or later (for Windows) OR Visual Studio Code + C#/.NET extension (available for Linux, macOS and Windows)
    - for editing and debugging
- Gurobi 8.1.1


## How to build

### From command line

- `dotnet build` builds all projects

### In Visual Studio

- open the `.sln` file
- build the projects (`Build` -> `Build Solution`)


## Project structure

The project consists of six projects:  

*Class libraries*: 

- `TemporalSourceCover.Core`: contains core graph classes used by `TemporalSourceCover`
- `TemporalSourceCover.Utilities`: contains utility classes  used by `TemporalSourceCover`
- `TemporalSourceCover.DatasetTransformation`: contains the transformer classes used by `TemporalSourceCover`
	- if you need to add a new transformer class, do it in this project, the existing types can be used as a reference on how to implement a new transformer
- `Assertions`: contains a simple assertion class which is used for debugging and correctness purpose in some classes

*Executable projects*:

- `TemporalSourceCover`: contains an implementation of the temporal source cover algorithm and data reduction rules
	- see below about how to run 
- `RandomTemporalGraphGenerator`: contains a random simple temporal graph generator based on the H model by Casteigts et. al (https://arxiv.org/abs/2011.03738)



## How to use (from command line)

- navigate to the root directory of the project (contains the `.sln` file)
- `dotnet run -p ./TemporalSourceCover/TemporalSourceCover.csproj <args>` builds and runs the main program, but you need to provide arguments like below
	- either `<path to graph file> <transformer class name> [--bidirectional] [--heuristic]` (`bidirectional` and `heuristic` are optional but need to be provided in this order if both are present)
		- can be repeated multiple times if the program should run on multiple graphs
	- or `in:<path to file>` where the input file contains the parameters as explained above
		- an example of an input file and some basic graphs is given in `/TemporalSourceCover/Datasets/examples/*`
		- paths to datasets are relative to the path of the input file
	- using `--bidirectional` an edge in both directions will be created for each edge in the dataset
	- using `--heuristic` will run the heuristics instead of the exact algorithm(s)
	- additional dotnet notes
		- you can optionally select a configuration with the `-c` (or `--configuration` parameter)
		- available configurations are `Debug`, `DebugVerify`, `Release`, and `ReleaseVerify`, default configuration is `Debug`
		- if you do not have Gurobi installed you should use the `Debug` oder `Release` configurations
		- if you want to build and run with Gurobi you likely have to reference the dotnet DLL again 
			- edit `/TemporalSourceCover/TemporalSourceCover.csproj` and change the provided hint path in line 38 to the path of the Gurobi .NET Standard DLL
        - you can provide a lot more arguments than shown, see https://docs.microsoft.com/de-de/dotnet/core/tools/ for more information
- you can simply run the example file using `dotnet run -p ./TemporalSourceCover/TemporalSourceCover.csproj in:./TemporalSourceCover/Datasets/examples/examples-<x>.in`
    - to run with Gurobi use `dotnet run -c ReleaseVerify -p ./TemporalSourceCover/TemporalSourceCover.csproj in:./TemporalSourceCover/Datasets/examples/examples-<x>.in`

## Additional notes

- you can also build the project(s) using `dotnet build` and then run the created `.dll` with `dotnet run`
	- if you want to run on a machine without dotnet installed you can use `dotnet publish`
	- read the official documentation for more information
- for the exact algorithms a timeout of 1 hour is used by default, you can change the timeout without recompiling by creating a file called `.timeout` next to the binary file (in `/TemporalSourceCover/bin/Debug/net5.0/<config>` if used with `dotnet run`)
	- the format of the time in the file *must* be `hours:minutes:seconds` (can be `25:30:30` for example)
	- for the heuristic algorithms a timeout of 30 minutes is used by default, you can change the timeout by creating a file called `.heuristic-timeout` next to the binary (see above)
- create a file `.algofeat` next to the binary to enable/disable the search-tree algorithm and/or the data reduction rules
	- the file has to contain `rrules` to enable the data reduction rules
	- the file has to contain `searchtree` to enable the search-tree algorithm
	- without the file both are enabled by default
- data reduction groups can be enabled/disabled using a `.rrules` file next to the binary
	- the file needs to contain `temporal` to apply the local rules 
	- the file needs to contain `reachability` to apply the global (reachability) rules
	- without the file both are enabled by default
