﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

namespace TemporalSourceCover.DatasetTransformation
{
    /// <summary>
    /// Reads and returns the edges of a dataset with "u v t" format, where u and v are strings and t is an integer value
    /// </summary>
    public sealed class NameNameTimeTransformer : IDatasetTransformer
    {
        private readonly Dictionary<string, long> _vertices = new ();
        private long _nextId = 0;

        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;
                yield return TransformLine(line);
            }
        }

        private DatasetEdge TransformLine(string line)
        {
            string[] values = Regex.Split(line, @"\s+").Where(s => s.Length > 0).ToArray();


            string uName = values[0];
            long u = _vertices.GetOrAdd(uName, () => _nextId++);

            string vName = values[1];
            long v = _vertices.GetOrAdd(vName, () => _nextId++);

            long t = long.Parse(values[2]);

            return new DatasetEdge(u, v, t);
        }
    }
}
