﻿using System.Collections.Generic;
using System.IO;

namespace TemporalSourceCover.DatasetTransformation
{
    /// <summary>
    /// Reads and returns the edges of a dataset with "u,v,t" format, where u, v and t are integer values
    /// </summary>
    public sealed class NetRepoUVTCsvTransformer : IDatasetTransformer
    {
        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;
                yield return TransformLine(line);
            }
        }

        private DatasetEdge TransformLine(string line)
        {
            string[] values = line.Split(',');

            long u = long.Parse(values[0]);
            long v = long.Parse(values[1]);
            long t = long.Parse(values[2]);

            return new DatasetEdge(u, v, t);
        }
    }
}
