﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

namespace TemporalSourceCover.DatasetTransformation
{
    /// <summary>
    /// Reads and returns the edges of a dataset with "t u v" format, where u, v and t are integer values
    /// </summary>
    public sealed class TUVTransformer : IDatasetTransformer
    {
        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;
                yield return TransformLine(line);
            }
        }

        private DatasetEdge TransformLine(string line)
        {
            string[] values = Regex.Split(line, @"\s+").Where(s => s.Length > 0).ToArray();

            long t = long.Parse(values[0]);
            long u = long.Parse(values[1]);
            long v = long.Parse(values[2]);

            return new DatasetEdge(u, v, t);
        }
    }
}
